﻿using ComPDFKitViewer.PdfViewer;
using PDF_Office.EventAggregators;
using PDF_Office.Helper;
using PDF_Office.Model.EditTools.Bates;
using PDF_Office.Model;
using PDF_Office.Properties;
using PDFSettings;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Regions;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Controls;
using System.Windows;
using Prism.Common;
using PDF_Office.Model.Dialog.HomePageToolsDialogs.HomePageBatchProcessing;
using ComPDFKit.PDFDocument;
using Microsoft.Office.Interop.Word;

namespace PDF_Office.ViewModels.Dialog.HomePageToolsDialogs.HomePageBatchProcessing.HomePageBates
{
    public class HomePageBatesTemplateListContentViewModel : BindableBase, INavigationAware
    {
        private HomePageBatesDialogViewModel homePageBatesDialogViewModel;
        private CPDFViewer PDFViewer;
        public IEventAggregator eventAggregator;
        public BatesInfo BatesInfo = new BatesInfo();
        public BatesHeaderFooterItem BatesItem = new BatesHeaderFooterItem();
        public int TemplateIndex = 0;

        private ObservableCollection<BatesHeaderFooterItem> batesModCollection = new ObservableCollection<BatesHeaderFooterItem>();
        public ObservableCollection<BatesHeaderFooterItem> BatesModCollection
        {
            get { return batesModCollection; }
            set
            {
                batesModCollection = value;
                RaisePropertyChanged();
            }
        }

        private Visibility _createTemplateVisible;
        public Visibility CreateTemplateVisible
        {
            get { return _createTemplateVisible; }
            set { SetProperty(ref _createTemplateVisible, value); }
        }

        public DelegateCommand AddTemplateCommand { get; set; }
        public DelegateCommand<object> DeleteTemplateItemCommand { get; set; }
        public DelegateCommand<object> EditTemplateItemCommand { get; set; }
        public DelegateCommand DeleteAllTemplateItemCommand { get; set; }
        public DelegateCommand<object> SelectTemplateItemCommand { get; set; }
        public DelegateCommand EnterCreateCommand { get; set; }

        public string Unicode = null;

        public HomePageBatesTemplateListContentViewModel(IEventAggregator eventAggregator)
        {
            this.eventAggregator = eventAggregator;
            Unicode = App.mainWindowViewModel.SelectedItem.Unicode;
            EnterCreateCommand = new DelegateCommand(EnterCreate);
            AddTemplateCommand = new DelegateCommand(AddTemplate);
            EditTemplateItemCommand = new DelegateCommand<object>(RequestEditTemplateItem);
            DeleteTemplateItemCommand = new DelegateCommand<object>(DeleteTemplateItem);
            DeleteAllTemplateItemCommand = new DelegateCommand(DeleteAllTemplateItem);
            SelectTemplateItemCommand = new DelegateCommand<object>(SelectTemplateItem);
        }

        private void InitBackgroundTemplateList()
        {
            if (Settings.Default.BatesTemplateList == null)
            {
                Settings.Default.BatesTemplateList = new BatesTemplateList();
            }
            GetBatesSource();
        }

        public void EnterCreate()
        {
            this.eventAggregator.GetEvent<EnterTemplateListOrCreateEvent>().Publish(new EnumTemplateListOrCreateUnicode { Unicode = Unicode, Status = EnumTemplateListOrCreate.StatusCreate });
        }

        public void AddTemplate()
        {
            this.eventAggregator.GetEvent<EnterTemplateListOrCreateEvent>().Publish(new EnumTemplateListOrCreateUnicode
            {
                Unicode = Unicode,
                Status = EnumTemplateListOrCreate.StatusCreate
            });
        }

        public void SaveEditedBatesTemplateItem(BatesHeaderFooterItemUnicode BatesItemunicode)
        {
            BatesHeaderFooterItem batesItem = BatesItemunicode.Status;

            Settings.Default.BatesTemplateList[TemplateIndex] = batesItem;
            Settings.Default.Save();

            GetBatesSource();
        }

        public void RequestEditTemplateItem(object e)
        {
            var control = e as Control;
            if (control == null)
            {
                return;
            }
            var template = control.DataContext as BatesHeaderFooterItem;
            if (template == null)
            {
                return;
            }
            TemplateIndex = Settings.Default.BatesTemplateList.IndexOf(template);
            template.listIndex = TemplateIndex;
            ConvertItemToInfo(template, ref BatesInfo);
            this.eventAggregator.GetEvent<EditBatesHeaderFooterTemplateItemEvent>().Publish(new BatesHeaderFooterItemUnicode
            {
                Unicode = Unicode,
                Type = "HomePageBatesType",
                Status = template
            });
        }

        public void DeleteTemplateItem(object e)
        {
            var btn = e as System.Windows.Controls.Button;
            if (btn == null)
            {
                return;
            }
            BatesItem = btn.DataContext as BatesHeaderFooterItem;
            if (BatesItem == null)
            {
                return;
            }

            Settings.Default.BatesTemplateList.Remove(BatesItem);
            Settings.Default.Save();
            BatesModCollection.Remove(BatesItem);
            GetBatesSource();
        }

        public void DeleteAllTemplateItem()
        {
            while (Settings.Default.BatesTemplateList.Count != 0)
            {
                int temp = 0;
                var template = Settings.Default.BatesTemplateList[temp];

                Settings.Default.BatesTemplateList.Remove(template);
                Settings.Default.Save();

                BatesModCollection.Remove(template);

            }
            GetBatesSource();
        }

        public void SelectTemplateItem(object e)
        {
            var listBox = e as ListBox;
            BatesHeaderFooterItem BatesItem = listBox.SelectedItem as BatesHeaderFooterItem;
            SendTemplateItemToDocument(BatesItem);
        }

        public void SendTemplateItemToDocument(BatesHeaderFooterItem template)
        {
            if (template != null)
            {
                ConvertItemToInfo(template, ref BatesInfo);
                homePageBatesDialogViewModel.BatesInfo = BatesInfo;
                for(int i =0; homePageBatesDialogViewModel.PDFDataTable.Rows.Count> i;i++) {

                    homePageBatesDialogViewModel.PDFCurrentDataTable.Rows[i]["FilePageRangeText"] = HomePageEditHelper.UpDataPageRange(CPDFDocument.InitWithFilePath(HomePageBatchProcessingDialogModel.FilePaths[i]).PageCount, BatesInfo.PageRange);
                    homePageBatesDialogViewModel.PDFCurrentDataTable.Rows[i]["FilePageRangeSelectIndex"] = BatesInfo.PageRangeIndex.ToString();
                    homePageBatesDialogViewModel.updateListview("待确定");
                }
            }
        }

        private void GetBatesSource()
        {
            List<BatesHeaderFooterItem> batesModColorTemplateList = new List<BatesHeaderFooterItem>();
            for (int temp = 0; temp < Settings.Default.BatesTemplateList.Count; temp++)
            {
                Settings.Default.BatesTemplateList[temp].ItemName = (temp + 1).ToString();
                batesModColorTemplateList.Add(Settings.Default.BatesTemplateList[temp]);

            }
            BatesModCollection = new ObservableCollection<BatesHeaderFooterItem>(batesModColorTemplateList);
            CheckTemplateListIsEmpty(batesModColorTemplateList);
        }

        public void ConvertItemToInfo(BatesHeaderFooterItem batesItem, ref BatesInfo batesInfo)
        {
            batesInfo.TextData = batesItem.TextData;
            batesInfo.ItemName = batesItem.ItemName;
            batesInfo.DigitNumber = batesItem.DigitNumber;
            batesInfo.Prefix = batesItem.Prefix;
            batesInfo.Suffix = batesItem.Suffix;
            batesInfo.StarPagetNumber = batesItem.StarPagetNumber;
            batesInfo.margin = batesItem.margin;
            batesInfo.PageRangeIndex = batesItem.PageRangeIndex;
            batesInfo.PageRange = batesItem.PageRange;
            //EditToolsHelper.GetPageRange(batesInfo.PageRangeIndex, PDFViewer.Document, ref batesInfo.PageRange, batesItem.PageRange);

        }
        private void CheckTemplateListIsEmpty(List<BatesHeaderFooterItem> batesTemplateList)
        {
            if (batesTemplateList.Count() == 0)
            {
                CreateTemplateVisible = Visibility.Visible;
            }
            else
            {
                CreateTemplateVisible = Visibility.Collapsed;
            }
        }

        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            return true;
        }

        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
        }

        public void OnNavigatedTo(NavigationContext navigationContext)
        {
            navigationContext.Parameters.TryGetValue<CPDFViewer>(ParameterNames.PDFViewer, out PDFViewer);
            navigationContext.Parameters.TryGetValue<HomePageBatesDialogViewModel>("ViewModel", out homePageBatesDialogViewModel);
            InitBackgroundTemplateList();
        }
    }
}
