﻿using ComPDFKit.PDFDocument;
using ComPDFKitViewer.PdfViewer;
using PDF_Office.CustomControl;
using PDF_Office.Model;
using PDF_Office.Model.Dialog.HomePageToolsDialogs.HomePageBatchProcessing;
using PDF_Office.Model.Dialog.ToolsDialogs.SaftyDialogs;
using Prism.Commands;
using Prism.Mvvm;
using Prism.Regions;
using Prism.Services.Dialogs;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Forms;
using DataTable = System.Data.DataTable;
using DialogResult = Prism.Services.Dialogs.DialogResult;
using Task = System.Threading.Tasks.Task;

namespace PDF_Office.ViewModels.Dialog.HomePageToolsDialogs.HomePageBatchProcessing
{
    public class HomePageRemoveDialogViewModel : BindableBase, INavigationAware
    {

        #region 参数和属性
        public IDialogService dialogs;

        private List<string> fileNames;

        private CPDFDocument tempDocument;

        public List<int> fileNamesView = new List<int>();

        private int fileNamesIndex = 0;

        private int FileNameNumber = 0;

        HomePageCompressDialogModel homePageCompressDialogModel = new HomePageCompressDialogModel();

        private DataTable pdfDataTable = new DataTable();
        public DataTable PDFDataTable
        {
            get { return pdfDataTable; }
            set
            {
                SetProperty(ref pdfDataTable, value);
            }
        }

        private DataTable pdfCurrentDataTable = new DataTable();
        public DataTable PDFCurrentDataTable
        {
            get { return pdfCurrentDataTable; }
            set
            {
                SetProperty(ref pdfCurrentDataTable, value);
            }
        }

        private string optionsRemoveGridIsEnabled = "True";
        public string OptionsRemoveGridIsEnabled
        {
            get
            {
                return optionsRemoveGridIsEnabled;
            }
            set
            {
                SetProperty(ref optionsRemoveGridIsEnabled, value);
            }
        }

        private string setOptionsRemoveGridIsEnabled = "True";
        public string SetOptionsRemoveGridIsEnabled
        {
            get
            {
                return setOptionsRemoveGridIsEnabled;
            }
            set
            {
                SetProperty(ref setOptionsRemoveGridIsEnabled, value);
            }
        }

        private string removeIsEnable = "False";
        public string RemoveIsEnable
        {
            get { return removeIsEnable; }
            set
            {
                SetProperty(ref removeIsEnable, value);
            }
        }

        private string selectFileName = "False";
        public string SelectFileName
        {
            get { return selectFileName; }
            set
            {
                SetProperty(ref selectFileName, value);
            }
        }

        private Visibility addFileVisibility = Visibility.Hidden;
        public Visibility AddFileVisibility
        {
            get { return addFileVisibility; }
            set
            {
                SetProperty(ref addFileVisibility, value);
                RaisePropertyChanged();

            }
        }

        private bool securityIsRemove = false;
        public bool SecurityIsRemove
        {
            get { return securityIsRemove; }
            set
            {
                SetProperty(ref securityIsRemove, value);
            }
        }

        private bool watermarkIsRemove = false;
        public bool WatermarkIsRemove
        {
            get { return watermarkIsRemove; }
            set
            {
                SetProperty(ref watermarkIsRemove, value);
            }
        }

        private bool backgroundIsRemove = false;
        public bool BackgroundIsRemove
        {
            get { return backgroundIsRemove; }
            set
            {
                SetProperty(ref backgroundIsRemove, value);
            }
        }

        private bool headerFooterIsRemove = false;
        public bool HeaderFooterIsRemove
        {
            get { return headerFooterIsRemove; }
            set
            {
                SetProperty(ref headerFooterIsRemove, value);
            }
        }

        private bool batesIsRemove = false;
        public bool BatesIsRemove
        {
            get { return batesIsRemove; }
            set
            {
                SetProperty(ref batesIsRemove, value);
            }
        }
        #endregion

        #region 委托声明
        public DelegateCommand ADDPDFCommand { get; set; }

        public DelegateCommand RemovePDFFileCommand { get; set; }

        public DelegateCommand ADDPDFFilesCommand { get; set; }

        public DelegateCommand OptionsRemoveCommand { get; set; }

        #endregion
        public HomePageRemoveDialogViewModel(IDialogService dialogService)
        {
            dialogs = dialogService;
            PDFCurrentDataTable.Columns.Add("FileState");
            PDFDataTable.Columns.Add("FileName");
            PDFDataTable.Columns.Add("FileSize");
            PDFDataTable.Columns.Add("FileState");
            ADDPDFCommand = new DelegateCommand(addpicture);
            ADDPDFFilesCommand = new DelegateCommand(addpicturefiles);
            RemovePDFFileCommand = new DelegateCommand(removepdffile);
            OptionsRemoveCommand = new DelegateCommand(OptionsRemove);
        }
        #region 逻辑函数

        private bool CheckHaveAllPermissions(CPDFDocument document)
        {
            CPDFPermissionsInfo permissionsInfo = document.GetPermissionsInfo();
            if (permissionsInfo.AllowsDocumentChanges &&
                permissionsInfo.AllowsPrinting &&
                permissionsInfo.AllowsHighQualityPrinting &&
                permissionsInfo.AllowsCopying &&
                //permissionsInfo.AllowsDocumentAssembly&&
                //TODO: 这个权限有问题
                permissionsInfo.AllowsCommenting)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        private DeleteSafetySettintgsModel.EnumNeedPassword CheckNeedPassword(CPDFDocument document)
        {
            if (document.IsLocked)
            {
                return DeleteSafetySettintgsModel.EnumNeedPassword.StatusVerifyPassword;
            }
            else { }
            if (document.IsEncrypted)
            {
                if (CheckHaveAllPermissions(document))
                {
                    return DeleteSafetySettintgsModel.EnumNeedPassword.StatusDirectDescrypt;
                }
                else
                {
                    return DeleteSafetySettintgsModel.EnumNeedPassword.StatusVerifyPassword;
                }
            }
            else
            {
                return DeleteSafetySettintgsModel.EnumNeedPassword.StatusCannotDecrypt;
            }
        }

        private void OptionsRemove()
        {
            OptionsRemoveGridIsEnabled = "False";
            //GC.KeepAlive(indexDelegate);
            fileNamesIndex = 0;
            foreach (var filename in fileNames)
            {
                PDFDataTable.Rows[fileNamesIndex]["FileState"] = "待完成";
                CPDFDocument document = CPDFDocument.InitWithFilePath(filename);
                
                if (document == null)
                {
                    PDFDataTable.Rows[fileNamesIndex]["FileState"] = "文件出错";
                    PDFCurrentDataTable.Rows[fileNamesIndex]["FileState"] = "文件出错";
                    continue;
                }
                FileInfo fileinfo = new FileInfo(filename);
                if (document.IsLocked)
                {
                    DialogParameters value = new DialogParameters();
                    value.Add(ParameterNames.PDFDocument, document);
                    dialogs.ShowDialog(DialogNames.VerifyPassWordDialog, value, e =>
                    {
                        if (e.Result == ButtonResult.OK)
                        {
                            if (e.Parameters.ContainsKey(ParameterNames.PassWord) && e.Parameters.GetValue<string>(ParameterNames.PassWord) != null)
                            {
                              
                            }
                        }
                    });

                    if (document.IsLocked)
                    {
                        //未成功解密文档时，释放Document对象，返回
                        document.Release();
                        PDFDataTable.Rows[fileNamesIndex]["FileState"] = "文件出错";
                        PDFCurrentDataTable.Rows[fileNamesIndex]["FileState"] = "文件出错";
                        continue;
                    }
                }
                if (WatermarkIsRemove)
                {
                    document.DeleteWatermarks();
                }
                if (BackgroundIsRemove)
                {
                    document.GetBackground().Clear();
                }
                if (HeaderFooterIsRemove)
                {
                    document.GetHeaderFooter().Clear();
                }
                if (BatesIsRemove)
                {
                    document.GetBates().Clear();
                }
                if (SecurityIsRemove)
                {
                    document.Descrypt(filename + "_Remove.pdf");

                }
                else
                {
                    document.WriteToFilePath(filename + "_Remove.pdf");
                }
                PDFDataTable.Rows[fileNamesIndex]["FileState"] = "完成";
                PDFCurrentDataTable.Rows[fileNamesIndex]["FileState"] = "完成";
                fileNamesIndex++;
                document.Release();
            }
            OptionsRemoveGridIsEnabled = "True";
        }
        #endregion

        #region 批量处理逻辑函数
        /// <summary>
        /// 添加PDF文件
        /// </summary>
        private void addpicture()
        {
            FileNameNumber = fileNames.Count;
            System.Windows.Forms.OpenFileDialog dlg = new System.Windows.Forms.OpenFileDialog();
            dlg.Multiselect = true;
            dlg.Filter = "PDF|*.pdf;*.PDF;";
            if (dlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                fileNames.AddRange(dlg.FileNames.ToList());
                RemoveExcess(ref fileNames);
                SetOptionsRemoveGridIsEnabled = "True";
                AddFileVisibility = Visibility.Collapsed;
                updateListview("待确定");
            }
        }

        public void addPDFFiles(string filename) {
            FileNameNumber = fileNames.Count;
            fileNames.Add(filename);
            RemoveExcess(ref fileNames);
            SetOptionsRemoveGridIsEnabled = "True";
            AddFileVisibility = Visibility.Collapsed;
            updateListview("待确定");
        }
        

        /// <summary>
        /// 删除重复的文件
        /// </summary>
        public void RemoveExcess(ref List<string> Filenames)
        {
            List<string> filenames = new List<string>();
            foreach (var fileName in Filenames)
            {
                if (!filenames.Contains(fileName))
                {
                    filenames.Add(fileName);
                }
            }
            Filenames.Clear();
            Filenames = filenames;
        }

        /// <summary>
        /// 添加PDF文件夹
        /// </summary>
        private void addpicturefiles()
        {
            FileNameNumber = fileNames.Count;
            FolderBrowserDialog dialog = new FolderBrowserDialog();
            dialog.Description = "请选择文件路径";
            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {

                string foldPath = dialog.SelectedPath;
                var apps = System.IO.Directory.GetFiles(foldPath);
                foreach (string app in apps)
                {
                    var fi = new FileInfo(app);
                    if (fi.Extension == ".pdf" || fi.Extension == ".PDF")
                    {
                        fileNames.Add(app);
                    }
                }
                RemoveExcess(ref fileNames);
                updateListview("待确定");
                SetOptionsRemoveGridIsEnabled = "True";
                AddFileVisibility = Visibility.Collapsed;
            }

        }
        /// <summary>
        /// 更新listview显示
        /// state 状态显示字符串
        /// </summary>
        public void updateListview(string state)
        {
            updateCurrentListview();
            DataTable pdfdatatable = new DataTable();
            pdfdatatable.Columns.Add("FileName");
            pdfdatatable.Columns.Add("FileSize");
            pdfdatatable.Columns.Add("FileState");
            int datatableindex = 0;
            foreach (var fileName in fileNames)
            {
                string file_all = fileName;
                FileInfo f = new FileInfo(file_all);
                string file_size = (((float)f.Length) / 1024).ToString() + " K";
                pdfdatatable.Rows.Add(f.Name, file_size, PDFCurrentDataTable.Rows[datatableindex]["FileState"]);
                datatableindex++;
            }
            PDFDataTable = pdfdatatable;
            HomePageBatchProcessingDialogModel.FilePaths = fileNames;

        }

        /// <summary>
        /// 更新Currentlistview显示
        /// state 状态显示字符串
        /// </summary>
        public void updateCurrentListview(string state = "待完成")
        {
            if (fileNames.Count >= FileNameNumber)
            {
                for (int i = 0; fileNames.Count - FileNameNumber > i; i++)
                {
                    PDFCurrentDataTable.Rows.Add(state);
                }
            }
            else
            {
                Reverseorder(ref fileNamesView);
                foreach (int filenamesview in fileNamesView)
                {
                    PDFCurrentDataTable.Rows.RemoveAt(filenamesview);
                }
            }
        }

        /// <summary>
        /// 逆序int类型集合
        /// </summary>
        public void Reverseorder(ref List<int> Numbers)
        {
            Numbers = Numbers.OrderBy(a => a).ToList();
            Numbers.Reverse();
        }

        /// <summary>
        /// 打开文件PDF
        /// </summary>
        public void openfiledialog()
        {

            foreach (int filenamesview in fileNamesView)
            {
                System.Diagnostics.Process.Start(fileNames[filenamesview]);
            }
        }

        /// <summary>
        /// 删除文件PDF
        /// </summary>
        public void removepdffile()
        {
            FileNameNumber = fileNames.Count;
            Reverseorder(ref fileNamesView);
            foreach (int filenamesview in fileNamesView)
            {
                //Trace.WriteLine(filenamesview);
                fileNames.Remove(fileNames[filenamesview]);
            }
            if (fileNames.Count < 1)
            {
                SetOptionsRemoveGridIsEnabled = "False";
                AddFileVisibility = Visibility.Visible;
            }
            updateListview("待确定");
        }
        public void removepdffile(int index)
        {
            fileNames.Remove(fileNames[index]);

            if (fileNames.Count < 1)
            {
                SetOptionsRemoveGridIsEnabled = "False";
                AddFileVisibility = Visibility.Visible;
            }

            updateListview("待确定");
        }

        public void PDFFileCount()
        {
            if (fileNames.Count == 0)
            {
                SetOptionsRemoveGridIsEnabled = "False";
                AddFileVisibility = Visibility.Visible;
            }
            else
            {
                SetOptionsRemoveGridIsEnabled = "True";
                AddFileVisibility = Visibility.Collapsed;
            }
        }
        #endregion

        #region 构架行为
        public void OnNavigatedTo(NavigationContext navigationContext)
        {
            List<string> filepath = new List<string>();
            navigationContext.Parameters.TryGetValue<List<string>>(ParameterNames.FilePath, out filepath);
            if (filepath != null)
            {

                fileNames = filepath;
                PDFFileCount();
                updateListview("待确定");
            }
        }

        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            return true;
        }

        public void OnNavigatedFrom(NavigationContext navigationContext)
        {

        }
        #endregion
    }
}
