﻿using PDF_Office.EventAggregators;
using PDF_Office.Model.Dialog.HomePageToolsDialogs.HomePagePrinter;
using Prism.Commands;
using Prism.Events;
using Prism.Mvvm;
using Prism.Regions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Management.Instrumentation;
using System.Windows.Controls;

namespace PDF_Office.ViewModels.Dialog.HomePageToolsDialogs.HomePagePrinter
{
    public class HomePagePrinterModSizeContentViewModel : BindableBase, INavigationAware
    {
        private IEventAggregator eventAggregator;
        private string Unicode = null;
        public string CurrentSelectedRdoName = "StatusAdaptiveRdo";

        private string _displayRatio = "100";
        public string DisplayRatio
        {
            get { return _displayRatio; }
            set { SetProperty(ref _displayRatio, value); }
        }

        private SizeInfo _sizeInfo;
        public SizeInfo SizeInfo
        {
            get { return _sizeInfo; }
            set { _sizeInfo = value; }
        }

        public DelegateCommand<object> SetModSizeStatusCommand { get; set; }
        public DelegateCommand SetDisplayRatioCommand { get; set; }

        //public DelegateCommand sendSizeInfoCommand;
        //public DelegateCommand SendSizeInfoCommand => this.SendSizeInfoCommand ?? (this.sendSizeInfoCommand = new DelegateCommand(() => this.eventAggregator.GetEvent<ModInfoSendEvent>().Publish(PrintMod)));

        public DelegateCommand SendSizeInfoCommand { get; set; }

        public HomePagePrinterModSizeContentViewModel(IEventAggregator eventAggregator)
        {
            this.eventAggregator = eventAggregator;
            SizeInfo = new SizeInfo();
            SizeInfo.EnumPrintMod = EnumPrintMod.StatusSize;
            SetModSizeStatusCommand = new DelegateCommand<object>(SetModSizeStatus);

            SetDisplayRatioCommand = new DelegateCommand(SetDisplayRatio);
        }

        public void SendSizeInfo()
        {
            if (SizeInfo != null)
            {
                this.eventAggregator.GetEvent<SendPrintSettingsModInfoEvent>().Publish(new PrintModInfoWithUnicode { printModInfo = SizeInfo, Unicode = this.Unicode});
            }
        }

        public void SetModSizeStatus(object e)
        {
            var rdo = e as RadioButton;
            if (rdo.Name == CurrentSelectedRdoName)
            {
                return;
            }
            else
            {
                CurrentSelectedRdoName = rdo.Name;
                if (rdo.Name == "StatusAdaptiveRdo")
                {
                    SizeInfo.EnumSizeType = EnumSizeType.StatusAdaptive;
                }
                else if (rdo.Name == "StatusActuralRdo")
                {
                    SizeInfo.EnumSizeType = EnumSizeType.StatusActural;
                }
                else
                {
                    SizeInfo.EnumSizeType = EnumSizeType.StatusCustomized;
                }
                if (SizeInfo.EnumSizeType == EnumSizeType.StatusAdaptive || SizeInfo.EnumSizeType == EnumSizeType.StatusActural || (SizeInfo.EnumSizeType == EnumSizeType.StatusCustomized)&& DisplayRatio != null)
                {
                    SendSizeInfo();
                }
            }
        }

        public void SetDisplayRatio()
        {
            int previousDisplayRatio = SizeInfo.DisplayRatio;
            SizeInfo.DisplayRatio = int.Parse(DisplayRatio);
            if (previousDisplayRatio != SizeInfo.DisplayRatio)
            {
                SendSizeInfo();
            }
        }


        public bool IsNavigationTarget(NavigationContext navigationContext)
        {
            return true;
        }

        public void OnNavigatedFrom(NavigationContext navigationContext)
        {
        }

        public void OnNavigatedTo(NavigationContext navigationContext)
        {
            navigationContext.Parameters.TryGetValue<string>("Unicode", out Unicode);
            SendSizeInfo();
        }
    }
}
